@@ -1,6 +1,6 @@
 
 /*
- *	@(#)gui.c	1.32 10/23/94
+ *	@(#)gui.c	1.49 7/11/95
  *
  *	Gui.c : It is intended that this file will contain as much
  *	as possible of the user-interface code, allowing privtool to
@@ -17,12 +17,18 @@
  *
  *			- Mark Grant (mark@unicorn.com) 29/6/94
  *
+ *	Linux compatibility changes by 
+ *		- David Summers (david@actsn.fay.ar.us) 6th July 1995
+ *
  */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#ifdef SYSV
+#include <unistd.h>
+#endif
 
 #include "def.h"
 #include "buffers.h"
@@ -30,6 +36,10 @@
 #include "mailrc.h"
 #include "gui.h"
 
+/* Maximum number of remailers to use */
+
+#define MAXMIX	6
+
 /* Local variables */
 
 static	int	security;
@@ -50,6 +60,21 @@ static	char	*print_args[] =
 	NULL
 };
 
+#ifndef NO_MIXMASTER
+static	char	*remail_args[32] =
+{
+	MIXEXEC,
+	"-c",
+#ifdef MIXMASTER_STDIN
+	"stdin",
+#else
+	NULL,
+#endif
+	"-f",
+	"-to"
+};
+#endif
+
 /* Titles for passphrase windows */
 
 static	char	sign_passphrase[]="Passphrase entry (Signing)";
@@ -68,7 +93,7 @@ int	deliver_flags;
 
 /* Program name */
 
-char	prog_name[] = "Privtool V0.81 BETA";
+char	prog_name[] = "Privtool V0.84a BETA";
 
 /* Set the description of the message for the message list */
 
@@ -334,7 +359,8 @@ m->flags &= ~(MESS_SIGNED|MESS_ENCRYPTED);
 
 			/* Call PGP to decrypt/verify */
 
-			i = decrypt_message(m->message,m->decrypted,
+			i = decrypt_message(message_contents(m),
+				m->decrypted,
 				m->signature,passphrase,FL_ASCII);
 
 			/* Destroy the passphrase if we got one */
@@ -358,9 +384,8 @@ m->flags &= ~(MESS_SIGNED|MESS_ENCRYPTED);
 
 			if (i == DEC_BAD_FILE) {
 				bad_file_notice (ERROR_READING);
-				clear_display_footer();
-				last_message_read = NULL;
-				return;
+				m->flags &= ~(MESS_ENCRYPTED|MESS_SIGNED);
+				goto display_plaintext;
 			}
 
 			/* Process the error and set flags */
@@ -421,11 +446,13 @@ m->flags &= ~(MESS_SIGNED|MESS_ENCRYPTED);
 			show_addkey ();
 	}
 	else {
+
+	display_plaintext:
 		/* Or display the plaintext message */
 
-		display_message_body(m->message);
+		display_message_body(message_contents (m));
 
-		if (buffer_contains_key (m->message))
+		if (buffer_contains_key (message_contents (m)))
 			show_addkey ();
 	}
 
@@ -546,6 +573,7 @@ void	quit_proc()
 	destroy_passphrase(TRUE);
 
 	shutdown_ui();
+	close_messages ();
 	close_pgplib();
 
 	exit(1);
@@ -833,7 +861,11 @@ char	**list;
 
 	/* Sort the list */
 
+#ifndef linux
 	qsort (list, i, sizeof (char *), strcmp);
+#else
+	qsort (list, i, sizeof (char *), (__compar_fn_t) strcmp);
+#endif
 
 	i = 1;
 	j = 1;
@@ -938,6 +970,96 @@ char	*s;
 	return s;
 }
 
+#ifndef NO_MIXMASTER
+static	setup_remail_args(addrs, subject, temp)
+
+char	*addrs, *subject;
+char	*temp;
+
+{
+	int	i, j;
+	int	l;
+	int	c;
+	FILE	*mix_fp;
+	char	mix_path[1024];
+	static	char	n[MAXMIX][5];
+	char	used[100];
+
+	i = 5;
+
+#ifndef MIXMASTER_STDIN
+	remail_args[2] = temp;
+#endif
+
+	/* Pass subject if neccesary */
+
+	remail_args[i++] = addrs;
+
+	remail_args[i++] = "-s";
+	if (subject && *subject) 
+		remail_args[i++] = subject;
+	else
+		remail_args[i++] = "None";
+
+	/* Open type2.list */
+
+	sprintf(mix_path, "%s/type2.list", MIXPATH);
+
+	mix_fp = fopen(mix_path, "rt");
+	if (!mix_fp)
+		return FALSE;
+
+	/* Count number of lines in type2.list */
+
+	l = 0;
+	while (!feof(mix_fp)) {
+		c = fgetc(mix_fp);
+		if (c == '\n')
+			l++;
+	}
+
+	fclose (mix_fp);
+
+	/* Ok, we now know how many remailers are available. We only
+	   select from first 100 */
+
+	if (!l)
+		return FALSE;
+
+	if (l > 100)
+		l = 100;
+
+	for (c = 0; c < l; c++)
+	 	used[c] = FALSE;
+
+	c = l;
+	if (c > MAXMIX)
+		c = MAXMIX;
+
+	remail_args[i++] = "-l";
+
+	/* This isn't absolutely secure, but it's not *that* important */
+
+	do {
+		do {
+#ifdef PGPTOOLS
+			j = our_randombyte() * 256 + our_randombyte();
+#else
+			j = random();
+#endif
+			j %= l;
+		} while (used[j]);
+
+		used[j] = TRUE;
+		sprintf(n[c], "%d", j + 1);
+
+		remail_args[i++] = n[c];
+	} while (--c);
+
+	return TRUE;
+}
+#endif
+
 void	deliver_proc()
 
 {
@@ -960,6 +1082,7 @@ void	deliver_proc()
 	int	i, j;
 	BUFFER	*full_list;
 	BUFFER	*cc_list;
+	char	*nym;
 
 	update_random();
 
@@ -1062,7 +1185,8 @@ try_again:
 
 			/* Get our key name for the routine */
 
-			key_name = cuserid(NULL);
+			key_name = nym = current_nym ();
+
 			if (s = find_pgpkey (key_name))
 				key_name = s;
 
@@ -1161,38 +1285,132 @@ try_again:
 
 		mail_message = new_buffer();
 
+		if (!(deliver_flags & DELIVER_REMAIL)) {
 #ifdef MAILER_LINE
-		sprintf(buff,"Mailer: %s\n",prog_name);
-		add_to_buffer(mail_message,buff,strlen(buff));
+			sprintf(buff,"Mailer: %s\n",prog_name);
+			add_to_buffer(mail_message,buff,strlen(buff));
 #endif
 
-		if (replying_message_id &&
-			replying_message_sender) {
-			sprintf(buff,"In-Reply-To: %s from \"%s\"\n",
-				replying_message_id,
-				replying_message_sender);
-			add_to_buffer (mail_message,buff,strlen(buff));
+			if (replying_message_id &&
+				replying_message_sender) {
+				sprintf(buff,"In-Reply-To: %s from \"%s\"\n",
+					replying_message_id,
+					replying_message_sender);
+				add_to_buffer (mail_message,buff,strlen(buff));
+			}
+
+			if (*subject) {
+				sprintf(buff,"Subject: %s\n",subject);
+				add_to_buffer(mail_message,buff,strlen(buff));
+			}
+
+			add_to_buffer (mail_message, "To: ", 4);
+			add_list_to_header (mail_message, addrs);
+
+			if (cc_addrs) {
+				add_to_buffer (mail_message, "Cc: ", 4);
+				add_list_to_header (mail_message, cc_addrs);
+			}
+
+#if 0
+			if (strcmp(nym, cuserid(0))) {
+				add_to_buffer (mail_message, "From: ", 6);
+				add_to_buffer (mail_message, nym, strlen(nym));
+			}
+#endif
+
+			add_to_buffer(mail_message,"\n",1);
 		}
 
-		if (*subject) {
-			sprintf(buff,"Subject: %s\n",subject);
-			add_to_buffer(mail_message,buff,strlen(buff));
-		}
-
-		add_to_buffer (mail_message, "To: ", 4);
-		add_list_to_header (mail_message, addrs);
-
-		if (cc_addrs) {
-			add_to_buffer (mail_message, "Cc: ", 4);
-			add_list_to_header (mail_message, cc_addrs);
-		}
-
-		add_to_buffer(mail_message,"\n",1);
 		add_to_buffer(mail_message,to_send->message,
 			to_send->length);
 
-		run_program(mail_args[0],mail_message->message,
-			mail_message->length,mail_args,NULL);
+		/* Do we remail or send direct ? */
+
+#ifndef NO_MIXMASTER
+		if (deliver_flags & DELIVER_REMAIL) {
+			char	*addr;
+#ifdef MIXMASTER_STDIN
+			char	*temp = NULL;
+#else
+			char	temp[1024];
+			FILE	*mix_fp;
+
+			sprintf(temp, "%s/temp.mix", MIXPATH);
+			mix_fp = fopen(temp, "wt");
+
+			if (!mix_fp)
+				goto remail_error_exit;
+
+			fwrite (mail_message->message, 
+				mail_message->length,
+				1, mix_fp);
+
+			fclose (mix_fp);
+#endif
+
+			i = 0;
+			while (addrs[i]) {
+				if (setup_remail_args(addrs[i], subject, temp)) {
+					run_program(remail_args[0],
+						mail_message->message,
+#ifdef MIXMASTER_STDIN
+						mail_message->length,
+#else
+						0,
+#endif
+						remail_args,
+						NULL);
+				}	
+				else {
+
+remail_error_exit:
+					remail_failed_notice_proc();
+					if (raw_message)
+						free_buffer (raw_message);
+
+					free_buffer(to_send);
+					free_buffer(mail_message);
+					free_buffer (full_list);
+					free (addrs);
+					if (cc_addrs)
+						free (cc_addrs);
+
+#ifndef MIXMASTER_STDIN
+					unlink (temp);
+#endif
+
+					return;
+				}
+
+				i++;
+			}
+
+			i = 0;
+			while (cc_addrs && cc_addrs[i]) {
+				if (setup_remail_args(cc_addrs[i], subject))
+					run_program(remail_args[0],
+						mail_message->message,
+						mail_message->length,
+						remail_args,
+						NULL);
+				else 
+					goto remail_error_exit;
+				i++;
+			}
+
+#ifndef MIXMASTER_STDIN
+			unlink (temp);
+#endif
+
+		}
+		else {
+#endif
+			run_program(mail_args[0],mail_message->message,
+				mail_message->length,mail_args,NULL);
+#ifndef NO_MIXMASTER
+		}
+#endif
 
 		if (deliver_flags & DELIVER_LOG)  {
 			char	*record;
@@ -1224,6 +1442,9 @@ try_again:
 		free_buffer (full_list);
 		free (addrs);
 
+		if (cc_addrs)
+			free (cc_addrs);
+
 		close_deliver_window();
 	}
 }
@@ -1776,8 +1997,9 @@ int	raw;
 				}
 			}
 			else {
-				add_to_buffer (out, m->message->message,
-					m->message->length);
+				add_to_buffer (out, 
+					message_contents(m)->message,
+					message_contents(m)->length);
 			}
 
 			add_to_buffer (out, "\n", 1);
