@@ -1,6 +1,6 @@
 
 /*
- *	@(#)mail_reader.c	1.32 12/16/94
+ *	@(#)mail_reader.c	1.35 7/11/95
  *
  *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
  *	The author assumes no liability for damages resulting from the 
@@ -16,6 +16,9 @@
  *
  *			- Mark Grant (mark@unicorn.com) 29/6/94
  *
+ *	Linux compatibility changes by 
+ *		- David Summers (david@actsn.fay.ar.us) 6th July 1995
+ *
  */
 
 #include <stdio.h>
@@ -48,11 +51,15 @@ static	int	read_only = FALSE;
 extern	char	default_mail_file [];
 extern	FILE	*mail_fp;
 
+#ifndef linux
 #ifdef SYSV
+#ifdef NON_ANSI
 void	usleep(n)
 
 int	n;
-
+#else
+void	usleep (int n)
+#endif
 {
 	struct	timeval	t;
 
@@ -62,6 +69,7 @@ int	n;
 	(void) select (0, NULL, NULL, NULL, &t);
 }
 #endif
+#endif
 
 /* Unfortunately, we have to cope with people who use very, very long lines */
 
@@ -140,7 +148,7 @@ int	n;
 	/* Firewall */
 
 	if (!b)
-		return NULL;
+		return 0;
 
 	/* Count the \n's */
 
@@ -414,7 +422,7 @@ MESSAGE	*m;
 
 	/* Check for Encrypted: PGP */
 
-	else if (!strcasecmp(line,"Encrypted: PGP",15)) {
+	else if (!strncasecmp(line,"Encrypted: PGP",15)) {
 		m->flags |= MESS_ENCRYPTED;
 	}
 }
@@ -427,6 +435,9 @@ MESSAGE	*m;
 #ifndef SYSV
 	struct	tm	t;
 #else
+#ifdef linux
+	time_t	tmp;
+#endif
 	struct	tm	*t;
 #endif
 
@@ -452,13 +463,19 @@ MESSAGE	*m;
 
 	return timegm (&t);
 #else
-	t = getdate (m->date);
+#ifdef linux
+#define getdate(x) (parsedate((x),NULL))
+	tmp = getdate (m->date);
+	t = localtime (&tmp);
+#else
+	t = (struct tm *)getdate (m->date);
 
 	if (t)
 		return mktime (t);
 	else
 		return 0;
 #endif
+#endif
 }
 
 MESSAGE	*message_from_message(m)
@@ -1122,6 +1139,7 @@ int	save_all;
 
 	if (!fp) {
 		printf("Fopen failed : errno = %d\n",errno);
+		unlink (lock_file);
 		return -1;
 	}
 
