@@ -1,7 +1,7 @@
 
 
 /*
- * @(#)mailrc.h	1.3 6/29/94
+ * @(#)mailrc.h	1.6 7/6/95
  *
  *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
  *	The author assumes no liability for damages resulting from the 
@@ -41,5 +41,10 @@ typedef struct {
 	MAILRC	*start;
 	MAILRC	*end;
 
+	int32	number;
+
 } LIST;
 
+extern	char	*nym_name();
+extern	char	*current_nym();
+
