@@ -1,6 +1,6 @@
 
 /*
- * @(#)main.c	1.19 12/16/94
+ * @(#)main.c	1.24 7/6/95
  *
  *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
  *	The author assumes no liability for damages resulting from the 
@@ -36,9 +36,12 @@ static	LIST	alias;
 static	LIST	pgpkey;
 static	LIST	kills_l;
 static	LIST	killu_l;
+static	LIST	nym_list;
 static	LIST	cfeed;
 static	LIST	ignore;
 
+static	char	*our_nym;
+
 static	int	security_level = DEFAULT_SECURITY;
 
 static	char	*strip_quotes[] = {
@@ -119,6 +122,12 @@ LIST	*l;
 MAILRC	*m;
 
 {
+	/* Update count of entries */
+
+	l->number++;
+
+	/* Add new entry */
+
 	if (l->end) {
 		l->end->next = m;
 		m->prev = l->end;
@@ -128,6 +137,27 @@ MAILRC	*m;
 		l->start = l->end = m;
 }
 
+/* Move a mailrc entry to the top of the list */
+
+static	move_to_top(l, m)
+
+LIST	*l;
+MAILRC	*m;
+
+{
+	if (l->start == m) 
+		return;
+
+	if (l->end == m)
+		l->end = m->prev;
+
+	m->prev = NULL;
+	m->next = l->start;
+
+	l->start->prev = m;
+	l->start = m;
+}
+
 static	add_pgpkey(s)
 
 char	*s;
@@ -260,6 +290,72 @@ char	*s;
 	add_entry(&killu_l,s);
 }
 
+static	add_nym (s)
+
+char	*s;
+
+{
+	add_entry(&nym_list, s);
+}
+
+static	default_nym (s)
+
+char	*s;
+
+{
+	MAILRC	*m;
+
+	m = nym_list.start;
+
+	while (m) {
+		if (!strcasecmp (m->name,s)) {
+			move_to_top (&nym_list, m);
+			return;
+		}
+		m = m->next;
+	}
+}
+
+int32	nym_count()
+
+{
+	return nym_list.number;
+}
+
+char	*nym_name(n)
+
+int	n;
+
+{
+	MAILRC	*m;
+
+	if (n > nym_list.number)
+		return NULL;
+
+	m = nym_list.start;
+
+	while (m && n) {
+		m = m->next;
+		n--;
+	}
+
+	return m->name;
+}
+
+char	*current_nym()
+
+{
+	return our_nym;
+}
+
+void	set_current_nym(s)
+
+char	*s;
+
+{
+	our_nym = s;
+}
+
 static	add_kills(s)
 
 char	*s;
@@ -410,6 +506,20 @@ FILE	*fp;
 				continue;
 			}
 
+			/* Pseudonym */
+
+			if (!strncasecmp(s, "pseudonym", 9)) {
+				add_nym (s+10);
+				continue;
+			}
+
+			/* Default nym */
+
+			if (!strncasecmp(s, "defnym", 6)) {
+				default_nym (s+7);
+				continue;
+			}
+
 			/* Encrypted feed */
 
 			if (!strncasecmp(s,"cfeed",5)) {
@@ -511,6 +621,9 @@ static	void	read_mailrc()
 
 	loc = getenv("MAILRC");
 
+	add_nym(cuserid(NULL));
+	our_nym = nym_name (0);
+
 	if (!loc) {
 
 		/* If that fails, try $HOME/.mailrc */
@@ -561,6 +674,8 @@ char	*argv[];
 	if (!pgp_path()) {
 		printf ("Can't find PGP at '%s', exiting !\n", pgp_exec);
 	}
+
+	srandom (time(0) + getpid());
 #endif
 
 	/* Set it here before we forget ! */
