@@ -1,5 +1,5 @@
 /*
- *	@(#)x.c	1.36 12/16/94
+ *	@(#)x.c	1.44 7/6/95
  *
  *	(c) Copyright 1993-1994 by Mark Grant. All right reserved.
  *	The author assumes no liability for damages resulting from the 
@@ -13,6 +13,9 @@
  *	
  *      Menu item for selecting Mail files added by
  *      	- Keith Paskett (keith.paskett@sdl.usu.edu) 8 Sep 1994
+ *
+ *	Linux compatibility changes by 
+ *		- David Summers (david@actsn.fay.ar.us) 6th July 1995
  */
 
 #include <stdlib.h>
@@ -21,7 +24,12 @@
 #include <sys/stat.h>
 #include <dirent.h>
 #include <sys/dirent.h>
+#ifndef linux
 #include <X11/Xos.h>
+#else
+#include <signal.h>
+#define SCO324
+#endif
 #ifndef MAXPATHLEN
 #include <sys/param.h>
 #endif /* MAXPATHLEN */
@@ -36,6 +44,9 @@
 #include <xview/font.h>
 #include <xview/scrollbar.h>
 #include <xview/notify.h>
+#ifdef linux
+#include <X11/Xos.h>
+#endif
 
 #include "def.h"
 #include "buffers.h"
@@ -50,6 +61,7 @@ static	Panel	top_panel;
 static	Panel	main_panel;
 static	Panel	display_panel;
 static	Panel_item	file_name_item;
+static	Panel_item	nym_item;
 static	Panel_item	include_item,deliver_item,header_item;
 static	Panel_item	clear_item, addkey_item;
 static	Panel_item	log_item,sign_item,encrypt_item;
@@ -315,6 +327,21 @@ int	f;
 			NULL);
 }
 
+#ifndef NO_MIXMASTER
+void	remail_failed_notice_proc()
+
+{
+	notice_prompt(top_panel, NULL,
+		NOTICE_MESSAGE_STRINGS, 
+			"Remail Failed !",
+			NULL,
+		NOTICE_BUTTON_YES, "Abort",
+		NOTICE_BUTTON_NO, "Ignore",
+		NULL);
+
+	update_random ();
+}
+#endif
 int	read_only_notice_proc ()
 
 {
@@ -696,6 +723,7 @@ x_setup_send_window()
 
 {
 	char	*log,*dontlog;
+	char	*nym;
 
 	/* Create the frame if not already done */
 
@@ -707,7 +735,7 @@ x_setup_send_window()
 			NULL);
 
 		deliver_panel = (Panel) xv_create(deliver_frame, PANEL,
-			XV_HEIGHT, 95,
+			XV_HEIGHT, 120,
 			PANEL_LAYOUT, PANEL_HORIZONTAL,
 			NULL);
 
@@ -725,14 +753,21 @@ x_setup_send_window()
 			NULL);
 
 		clear_item = xv_create (deliver_panel, PANEL_BUTTON,
-			PANEL_LABEL_STRING, "Include",
+			PANEL_LABEL_STRING, "Clear",
 			NULL);
 
 		log_item = xv_create (deliver_panel, PANEL_CHECK_BOX,
 			PANEL_LABEL_STRING, "Options",
 			PANEL_CHOOSE_ONE, FALSE,
-			PANEL_CHOICE_STRINGS, "Sign", "Encrypt", "Log", "Raw", NULL,
+			PANEL_CHOICE_STRINGS, "Sign", "Encrypt", "Log", 
+				"Raw", 
+#ifndef NO_MIXMASTER
+				"Remail", 
+#endif
+				NULL,
 			PANEL_NOTIFY_PROC, options_proc,
+			XV_X, 0,
+			XV_Y, 90,
 			NULL);
 
 		send_to_item = (Panel_item) xv_create(deliver_panel,
@@ -762,7 +797,7 @@ x_setup_send_window()
 
 		deliver_body_window = (Textsw) xv_create(deliver_frame, TEXTSW,
 			XV_HEIGHT, 288,
-			XV_Y, 95,
+			XV_Y, 120,
 			TEXTSW_MEMORY_MAXIMUM,1000000,
 			NULL);
 
@@ -781,12 +816,20 @@ x_setup_send_window()
 
 	deliver_flags = DELIVER_SIGN|DELIVER_ENCRYPT;
 
+	/* Default to remail if using a nym */
+
+	nym = current_nym();
+	if (strcmp(nym, cuserid(0))) 
+		deliver_flags |= DELIVER_REMAIL;
+
 	/* Clear the bits if told to by mailrc */
 
 	if (find_mailrc("nodefaultsign"))
 		deliver_flags &= ~DELIVER_SIGN;
 	if (find_mailrc("nodefaultencrypt"))
 		deliver_flags &= ~DELIVER_ENCRYPT;
+	if (find_mailrc("nodefaultremail"))
+		deliver_flags &= ~DELIVER_REMAIL;
 
 	/* Check mailrc for logging options */
 
@@ -886,6 +929,76 @@ static	reply_sender_no_include ()
 		TEXTSW_FIRST_LINE,0,NULL);
 }
 
+static	char	begin_forward[] = "-- Begin forwarded message ---\n";
+static	char	end_forward[] = "-- End forwarded message ---\n";
+
+static	forward_message ()
+
+{
+	int	i;
+	byte	*mess;
+	byte	*m;
+	BUFFER	*b;
+	char	subject[256];
+	char	*c;
+
+	update_random();
+
+	if (!last_message_read)
+		return;
+
+	set_reply (last_message_read);
+
+	setup_send_window();
+	
+	xv_set (send_cc_item,
+		PANEL_VALUE, "",
+		NULL);
+	
+	if (c = last_message_read->subject) {
+		c += strlen (c) - 6;
+		if (strncasecmp(last_message_read->subject,"(fwd)",5)) {
+			sprintf(subject,"%s (fwd)\n",
+				last_message_read->subject);
+		}
+		else
+			strcpy(subject,last_message_read->subject);
+
+		xv_set(send_subject_item,
+			PANEL_VALUE, subject,
+			NULL);
+	}
+	else
+		xv_set(send_subject_item,
+			PANEL_VALUE,"",
+			NULL);
+
+	if (last_message_read->decrypted)
+		b = last_message_read->decrypted;
+	else
+		b = message_contents(last_message_read);
+
+	textsw_insert(deliver_body_window,
+		begin_forward,
+		strlen(begin_forward));
+
+	textsw_insert(deliver_body_window,
+		b->message,
+		b->length);
+
+	textsw_insert(deliver_body_window,
+		end_forward,
+		strlen(end_forward));
+
+	xv_set(send_to_item,
+		PANEL_VALUE,"",
+		NULL);
+
+	xv_set(deliver_body_window,
+		TEXTSW_INSERTION_POINT,0,
+		TEXTSW_FIRST_LINE,0,NULL);
+}
+
 /* Reply_to sender and include message */
 
 static	reply_to_sender()
@@ -1785,13 +1898,28 @@ char *path;
         NULL);
 }
 
+static 	void	nym_notify (menu, menu_item)
+
+Menu    menu;
+Menu_item       menu_item;
+
+{
+	char	*nym;
+
+	nym = (char *)xv_get (menu_item, MENU_STRING);
+	set_current_nym (nym);
+
+	xv_set (nym_item, PANEL_READ_ONLY, FALSE, NULL);
+	xv_set (nym_item, PANEL_VALUE, nym, NULL);
+	xv_set (nym_item, PANEL_READ_ONLY, TRUE, NULL);
+}
 
 void	setup_ui(level)
 
 int	level;
 
 {
-	int	i, slen;
+	int	i, slen, l;
 	MESSAGE	*m;
 	GC	gc;
 	Display	*dpy;
@@ -1925,9 +2053,24 @@ int	level;
 		PANEL_ITEM_MENU, button_menu,
 		NULL);
 
+	button_menu = (Menu) xv_create (NULL, MENU,
+		MENU_ITEM, 
+			MENU_STRING, "New",
+			MENU_NOTIFY_PROC, send_message,
+			NULL,
+		MENU_ITEM, 
+			MENU_STRING, "Reply",
+			MENU_NOTIFY_PROC, reply_to_sender,
+			NULL,
+		MENU_ITEM,
+			MENU_STRING, "Forward",
+			MENU_NOTIFY_PROC, forward_message,
+			NULL,
+		NULL);
+
 	(void) xv_create (top_panel, PANEL_BUTTON,
 		PANEL_LABEL_STRING, "Compose",
-		PANEL_NOTIFY_PROC, send_message,
+		PANEL_ITEM_MENU, button_menu,
 		NULL);
 
 	button_menu = (Menu) xv_create (NULL, MENU,
@@ -2109,6 +2252,28 @@ int	level;
 		PANEL_ITEM_MENU, files_menu[2],
 		NULL);
 
+	button_menu = (Menu) xv_create(NULL, MENU, NULL);
+
+	l = nym_count ();
+	for (i = 0 ; i < l; i++) {
+		mi = (Menu_item) xv_create (XV_NULL, MENUITEM,
+			MENU_STRING, nym_name (i),
+			MENU_NOTIFY_PROC, nym_notify, 
+			NULL);
+		xv_set (button_menu, MENU_APPEND_ITEM, mi);
+	}
+
+	(void) xv_create (top_panel, PANEL_ABBREV_MENU_BUTTON,
+		PANEL_LABEL_STRING, "Nym:",
+		PANEL_ITEM_MENU, button_menu,
+		NULL);
+
+	nym_item = (Panel_item) xv_create (top_panel, PANEL_TEXT,
+		PANEL_VALUE_DISPLAY_LENGTH, 12,
+		PANEL_VALUE, nym_name (0),
+		PANEL_READ_ONLY, TRUE,
+		NULL);
+
 	list_canvas = (Canvas) xv_create(main_frame,
 		CANVAS,
 		CANVAS_AUTO_SHRINK, FALSE,
@@ -2130,6 +2295,13 @@ int	level;
 			XV_FONT);
 	}
 
+	/* Oh shit, we failed to find a font at all ! */
+
+	if (!list_font) {
+		printf("Failed to load suitable font !\n");
+		exit (1);
+	}
+
 	dpy = (Display *) xv_get(main_frame, XV_DISPLAY);
 
 	gcvalues.font = xv_get(list_font, XV_XID);
